local utils, idTweaksUtils
utils = {
  ["function"] = {
    id = function(x)
      return x
    end,
    apply = function(fn, a)
      return function(...)
        return fn(a, ...)
      end
    end
  },
  string = {
    join = function(s, t)
      return table.concat(t, s)
    end,
    split = function(sep, str)
      local o = { }
      string.gsub(str, (string.format("([^%s]+)", sep)), utils["function"].apply(table.insert, o))
      return o
    end
  },
  table = {
    list = {
      map = function(fn, t)
        local _accum_0 = { }
        local _len_0 = 1
        for i, v in ipairs(t) do
          _accum_0[_len_0] = fn(v, i)
          _len_0 = _len_0 + 1
        end
        return _accum_0
      end,
      filter = function(fn, t)
        local _accum_0 = { }
        local _len_0 = 1
        for i, v in ipairs(t) do
          if fn(v, i) then
            _accum_0[_len_0] = v
            _len_0 = _len_0 + 1
          end
        end
        return _accum_0
      end,
      reduce = function(fn, t, a)
        for i, v in ipairs(t) do
          a = fn(a, v, i)
        end
        return a
      end,
      each = function(fn, t)
        for i, v in ipairs(t) do
          fn(v, i)
        end
      end,
      index_of = function(v, t)
        for i, x in ipairs(t) do
          if x == v then
            return i
          end
        end
        return false
      end,
      contains = function(v, t)
        for i, x in ipairs(t) do
          if v == x then
            return true
          end
        end
        return false
      end,
      chunk = function(s, t)
        local o = { }
        local c = { }
        for i, v in ipairs(t) do
          if s == table.getn(c) then
            table.insert(o, c)
            c = { }
          end
          table.insert(c, v)
        end
        return o
      end,
      concat = function(t, ...)
        return utils.table.list.reduce(function(a, v)
          if (type(v)) == "table" then
            utils.table.list.each((function(x)
              return table.insert(a, x)
            end), v)
          else
            table.insert(a, v)
          end
          return a
        end, {
          ...
        }, utils.table.list.map(utils["function"].id, t))
      end
    },
    map = {
      get = function(k, t)
        local parts = utils.string.split(".", k)
        if #parts > 1 then
          local first = parts[1]
          table.remove(parts, 1)
          local rest = utils.string.join(".", parts)
          return utils.table.map.get(rest, t[first])
        else
          return t[k]
        end
      end,
      set = function(v, k, t)
        local parts = utils.string.split(".", k)
        if #parts > 1 then
          local first = parts[1]
          table.remove(parts, 1)
          local rest = utils.string.join(".", parts)
          return utils.table.map.set(v, rest, t[first])
        else
          t[k] = v
        end
      end,
      map = function(fn, t)
        local _tbl_0 = { }
        for k, v in pairs(t) do
          _tbl_0[k] = fn(v, k)
        end
        return _tbl_0
      end,
      filter = function(fn, t)
        local _tbl_0 = { }
        for k, v in pairs(t) do
          if fn(v, k) then
            _tbl_0[k] = v
          end
        end
        return _tbl_0
      end,
      reduce = function(fn, t, a)
        for k, v in pairs(t) do
          a = fn(a, v, k)
        end
        return a
      end,
      each = function(fn, t)
        for k, v in pairs(t) do
          fn(v, k)
        end
      end,
      clone = function(t)
        local _tbl_0 = { }
        for k, v in pairs(t) do
          _tbl_0[k] = v
        end
        return _tbl_0
      end,
      clone_deep = function(t)
        local _tbl_0 = { }
        for k, v in pairs(t) do
          _tbl_0[k] = (((type(v)) ~= "table") and v or utils.table.map.clone_deep(v))
        end
        return _tbl_0
      end,
      reverse = function(t)
        local _tbl_0 = { }
        for k, v in pairs(t) do
          _tbl_0[v] = k
        end
        return _tbl_0
      end,
      merge = function(...)
        return utils.table.list.reduce((function(a, x)
          utils.table.map.each((function(v, k)
            a[k] = v
          end), x)
          return a
        end), {
          ...
        }, { })
      end,
      keys = function(t)
        local _accum_0 = { }
        local _len_0 = 1
        for k, v in pairs(t) do
          _accum_0[_len_0] = k
          _len_0 = _len_0 + 1
        end
        return _accum_0
      end,
      values = function(t)
        local _accum_0 = { }
        local _len_0 = 1
        for k, v in pairs(t) do
          _accum_0[_len_0] = v
          _len_0 = _len_0 + 1
        end
        return _accum_0
      end
    }
  },
  db = {
    get_db_option = function(k, t)
      local parts = utils.string.split(".", k)
      if #parts <= 1 then
        return t[k]
      else
        local first = parts[1]
        table.remove(parts, 1)
        local rest = utils.string.join(".", parts)
        return utils.db.get_db_option(rest, t[first].args)
      end
    end,
    set_db_option = function(v, k, t)
      local parts = utils.string.split(".", k)
      if #parts <= 1 then
        t[k] = v
      else
        local first = parts[1]
        table.remove(parts, 1)
        local rest = utils.string.join(".", parts)
        return utils.db.set_db_option(v, rest, t[first].args)
      end
    end
  }
}
idTweaksUtils = utils
_G.idTweaksUtils = utils
